# -*- encoding: utf-8 -*-
#原画対照表
require 'spec_helper'

describe ProviderOriginalPicture do
  before do
    @admin = FactoryGirl.create :admin
    @sp = FactoryGirl.create :system_picture
    @lg = FactoryGirl.create :license_group
    @license = FactoryGirl.create :license, :license_group_id => @lg.id, :system_picture_id => @sp.id
    @user = FactoryGirl.create :user_yas
    @author = FactoryGirl.create :author, :user_id => @user.id
    @artist = FactoryGirl.create :artist, :author_id => @author.id
    @other_user = FactoryGirl.create :user_yas
    @other_author = FactoryGirl.create :author, :user_id => @other_user.id
    @other_artist = FactoryGirl.create :artist, :author_id => @other_author.id
    @provider_status = FactoryGirl.create :provider_status
    @provider = FactoryGirl.create :provider, :provider_status_id => @provider_status.id
  end
  describe '検証に於いて' do
    before do
      @op = FactoryGirl.create :original_picture, :artist_id => @artist.id
      @pop = FactoryGirl.build :provider_original_picture, :provider_id => @provider.id, :providers_original_picture_id => 2, :demanders_original_picture_id => @op.id
    end
    
    context 'オーソドックスなデータのとき' do
      it '下限データが通る' do
        @pop.should be_valid
      end
      it '上限データが通る' do
        @pop.should be_valid
      end
    end
    
    context 'provider_idを検証するとき' do
      it 'nullなら失敗する' do
        @pop.provider_id = nil
        @pop.should_not be_valid
      end
      it '数値でなければ失敗する' do
        @pop.provider_id = 'a'
        @pop.should_not be_valid
      end
      it '存在する貸手でなければ失敗する' do
        @pop.provider_id = 0
        @pop.should_not be_valid
      end
    end
    context 'providers_original_picture_idを検証するとき' do
      it 'nullなら失敗する' do
        @pop.providers_original_picture_id = nil
        @pop.should_not be_valid
      end
      it '数値でなければ失敗する' do
        @pop.providers_original_picture_id = 'a'
        @pop.should_not be_valid
      end
    end
    context 'demanders_original_picture_idを検証するとき' do
      it 'nullなら失敗する' do
        @pop.demanders_original_picture_id = nil
        @pop.should_not be_valid
      end
      it '数値でなければ失敗する' do
        @pop.demanders_original_picture_id = 'a'
        @pop.should_not be_valid
      end
    end
  end
  
  describe '対照表取得に於いて' do
    before do
      @op = FactoryGirl.create :original_picture, :artist_id => @artist.id
      @pop = FactoryGirl.create :provider_original_picture, :provider_id => @provider.id, :providers_original_picture_id => 2, :demanders_original_picture_id => @op.id
    end
    it '取得した対照表を返す' do
      r = ProviderOriginalPicture.get_one @provider.id, 2
      r.should be_true
    end
    it '該当する対照表が存在しなかったら、新規に興した対照表オブジェクトを返す' do
      r = ProviderOriginalPicture.get_one @provider.id, 1
      r.should be_a_new ProviderOriginalPicture
      r = ProviderOriginalPicture.get_one 0, 2
      r.should be_a_new ProviderOriginalPicture
    end
  end
  
  describe '貸手側原画同期に於いて' do
    before do
      @op = FactoryGirl.create :original_picture, :artist_id => @artist.id
      @pop = FactoryGirl.create :provider_original_picture, :provider_id => @provider.id, :providers_original_picture_id => 2, :demanders_original_picture_id => @op.id
      @newpop = ProviderOriginalPicture.new
    end
    context '自身に原画がリンクしてないとき' do
      before do
      end
      it '原画オブジェクトを新規に興す' do
        lambda {
          r = @newpop.modify_original_picture @op.attributes
        }.should change OriginalPicture, :count
      end
    end
    context '自身に原画がリンクしているとき' do
      it '自身から原画を取得する' do
        lambda {
          r = @pop.modify_original_picture @op.attributes
        }.should_not change OriginalPicture, :count
      end
    end
    it '貸手側原画カラム値からカラム値をセットする 原画オブジェクトを保存する' do
      r = @pop.modify_original_picture :filesize => 3346
      @op.reload
      @op.filesize.should eq 3346
    end
    it '原画オブジェクトを返す' do
      r = @pop.modify_original_picture @op.attributes
      r.should eq @op
    end
  end

  describe '貸手側実素材同期に於いて' do
    before do
      @op = FactoryGirl.create :original_picture, :artist_id => @artist.id
      @p = FactoryGirl.create :picture, :artist_id => @artist.id, :license_id => @license.id, :revision => 0,
        :original_picture_id => @op.id
      @p2 = FactoryGirl.build :picture, :artist_id => @artist.id, :license_id => @license.id, :revision => 1,
        :original_picture_id => @op.id
    end
    context '実素材が取得できないとき' do
      before do
      end
      it '実素材オブジェクトを新規に興す' do
        r = ProviderOriginalPicture.modify_picture @p2.attributes
        r.should be_a_new Picture
      end
    end
    context '実素材が取得できるとき' do
      it '実素材に原画idRevision取得を依頼して、実素材を取得している' do
        r = ProviderOriginalPicture.modify_picture @p.attributes
        r.should_not be_a_new Picture
        r.should eq @p
      end
    end
    it '実素材にカラム値をセットする' do
      r = ProviderOriginalPicture.modify_picture({:original_picture_id => @p.id, :revision => 0, :filesize => 3346})
      r.filesize.should eq 3346
    end
    it '実素材を返す' do
      r = ProviderOriginalPicture.modify_picture @p.attributes
      r.should eq @p
    end
  end

  describe '貸手側素材同期に於いて' do
    before do
      @op = FactoryGirl.create :original_picture, :artist_id => @artist.id
      @p = FactoryGirl.create :picture, :artist_id => @artist.id, :license_id => @license.id, :revision => 0,
        :original_picture_id => @op.id
      @rp = FactoryGirl.create :resource_picture, :artist_id => @artist.id, :original_picture_id => @op.id, :license_id => @license.id, :picture_id => @p.id
      @attr = FactoryGirl.attributes_for :resource_picture, :artist_id => @artist.id, :original_picture_id => 111, :license_id => @license.id, :picture_id => @p.id, :filesize => 36225
    end
    context '素材が取得できないとき' do
      before do
      end
      it '素材オブジェクトを新規に興す' do
        r = ProviderOriginalPicture.modify_resource_picture @attr
        r.should be_a_new ResourcePicture
      end
    end
    context '素材が取得できるとき' do
      it '素材に原画id取得を依頼して、素材を取得している' do
        r = ProviderOriginalPicture.modify_resource_picture @rp.attributes
        r.should_not be_a_new ResourcePicture
        r.should eq @rp
      end
    end
    it '素材にカラム値をセットする' do
      r = ProviderOriginalPicture.modify_resource_picture @attr
      r.filesize.should eq 36225
    end
    it '素材を返す' do
      r = ProviderOriginalPicture.modify_resource_picture @rp.attributes
      r.should eq @rp
    end
  end
  
  describe '絵師id置換に於いて' do
    before do
      @pa = FactoryGirl.create :provider_artist, :provider_id => @provider.id, :providers_artist_id => 5, :demanders_artist_id => @artist.id
      @pa2 = FactoryGirl.create :provider_artist, :provider_id => @provider.id, :providers_artist_id => 3, :demanders_artist_id => @other_artist.id
    end
    context 'つつがなく終わるとき' do
      before do
      end
      it 'trueを返す' do
        @attr5 = {'artist_id' => 5}
        r = ProviderOriginalPicture.replace_artist @provider.id, @attr5
        r.should be_true
      end
      it 'カラム値の絵師idを借手側の絵師のidで置き換える' do
        @attr5 = {'artist_id' => 5}
        r = ProviderOriginalPicture.replace_artist @provider.id, @attr5
        @attr5['artist_id'].should eq @artist.id
        @attr3 = {'artist_id' => 3}
        r = ProviderOriginalPicture.replace_artist @provider.id, @attr3
        @attr3['artist_id'].should eq @other_artist.id
      end
    end
    context '借手側の絵師がなかったとき' do
      before do
      end
      it 'falseを返す' do
        @attr = {'artist_id' => 1}
        r = ProviderOriginalPicture.replace_artist @provider.id, @attr
        r.should be_false
      end
    end
  end
  
  describe 'ライセンスid置換に於いて' do
    before do
      @other_license = FactoryGirl.create :license, :license_group_id => @lg.id, :system_picture_id => @sp.id, :url => 'http://other.license', :name => 'other_license'
      @pl = FactoryGirl.create :provider_license, :provider_id => @provider.id, :providers_license_id => 4, :demanders_license_id => @license.id
      @pl2 = FactoryGirl.create :provider_license, :provider_id => @provider.id, :providers_license_id => 2, :demanders_license_id => @other_license.id
    end
    context 'つつがなく終わるとき' do
      before do
      end
      it 'trueを返す' do
        @attr5 = {'license_id' => 4}
        r = ProviderOriginalPicture.replace_license @provider.id, @attr5
        r.should be_true
      end
      it 'カラム値のライセンスidを借手側のライセンスのidで置き換える' do
        @attr4 = {'license_id' => 4}
        r = ProviderOriginalPicture.replace_license @provider.id, @attr4
        @attr4['license_id'].should eq @license.id
        @attr2 = {'license_id' => 2}
        r = ProviderOriginalPicture.replace_license @provider.id, @attr2
        @attr2['license_id'].should eq @other_license.id
      end
    end
    context '借手側のライセンスがなかったとき' do
      before do
      end
      it 'falseを返す' do
        @attr = {'license_id' => 1}
        r = ProviderOriginalPicture.replace_license @provider.id, @attr
        r.should be_false
      end
    end
  end
  
  describe '実素材id置換に於いて' do
    before do
      @op = FactoryGirl.create :original_picture, :artist_id => @artist.id
      @p = FactoryGirl.create :picture, :original_picture_id => @op.id, :license_id => @license.id, :artist_id => @artist.id
      @rpattr = {:original_picture_id => @op.id, :picture_id => 3, :license_id => @license.id, :artist_id => @artist.id, :artist_name => 'tester', 
        :credit => '{}', :settings => {:reverse => 1, :gif_convert => 1}.to_json.to_s}
    end
    context '事前チェックする' do
      before do
        Picture.stub(:head).with(any_args).and_return(@p)
      end
      it '実素材にhead取得を問い合わせている' do
        Picture.should_receive(:head).with(any_args).exactly(1)
        r = ProviderOriginalPicture.replace_picture @provider.id, @rpattr
      end
    end
    context 'つつがなく終わるとき' do
      before do
        Picture.stub(:head).with(any_args).and_return(@p)
      end
      it 'trueを返す' do
        r = ProviderOriginalPicture.replace_picture @provider.id, @rpattr
        r.should be_true
      end
      it 'カラム値の実素材idを借手側の実素材のidで置き換える' do
        r = ProviderOriginalPicture.replace_picture @provider.id, @rpattr
        @rpattr['picture_id'].should eq @p.id
      end
    end
    context '借手側の実素材がなかったとき' do
      before do
        Picture.stub(:head).with(any_args).and_return(nil)
      end
      it 'falseを返す' do
        r = ProviderOriginalPicture.replace_picture @provider.id, @rpattr
        r.should be_false
      end
    end
  end
  
  describe '原画id置換に於いて' do
    before do
      @op = FactoryGirl.create :original_picture, :artist_id => @artist.id
      @other_op = FactoryGirl.create :original_picture, :artist_id => @artist.id
      @pop = FactoryGirl.create :provider_original_picture, :provider_id => @provider.id, :providers_original_picture_id => 6, :demanders_original_picture_id => @op.id
      @pop2 = FactoryGirl.create :provider_original_picture, :provider_id => @provider.id, :providers_original_picture_id => 9, :demanders_original_picture_id => @other_op.id
    end
    context 'つつがなく終わるとき' do
      before do
      end
      it 'trueを返す' do
        @attr6 = {'original_picture_id' => 6}
        r = ProviderOriginalPicture.replace_original_picture @provider.id, @attr6
        r.should be_true
      end
      it 'カラム値の原画idを借手側の原画のidで置き換える' do
        @attr6 = {'original_picture_id' => 6}
        r = ProviderOriginalPicture.replace_original_picture @provider.id, @attr6
        @attr6['original_picture_id'].should eq @op.id
        @attr9 = {'original_picture_id' => 9}
        r = ProviderOriginalPicture.replace_original_picture @provider.id, @attr9
        @attr9['original_picture_id'].should eq @other_op.id
      end
    end
    context '借手側の原画がなかったとき' do
      before do
      end
      it 'falseを返す' do
        @attr = {'original_picture_id' => 1}
        r = ProviderOriginalPicture.replace_original_picture @provider.id, @attr
        r.should be_false
      end
    end
  end
  
  describe '素材停止に於いて' do
    before do
      @op = FactoryGirl.create :original_picture, :artist_id => @artist.id
      @p = FactoryGirl.create :picture, :original_picture_id => @op.id, :license_id => @license.id, :artist_id => @artist.id
      rpattr = {:original_picture_id => @op.id, :picture_id => @p.id, :license_id => @license.id, :artist_id => @artist.id, :artist_name => 'tester', 
        :credit => '{}', :settings => {:reverse => 1, :gif_convert => 1}.to_json.to_s}
      @rp = FactoryGirl.create :resource_picture, rpattr
    end
    context '貸手側原画のカラム値の停止日時が設定されているとき' do
      before do
        @pop = ProviderOriginalPicture.create :provider_id => @provider.id, :providers_original_picture_id => 1, :demanders_original_picture_id => @op.id
        @stopped_op = FactoryGirl.build :original_picture, :artist_id => @artist.id, :stopped_at => Time.now + 400
        @stopped_op_attr = @stopped_op.attributes
        ResourcePicture.any_instance.stub(:unpublish).with(any_args).and_return(true)
      end
      context '対照表から素材を取得できるとき' do
        before do
          @stopped_op = FactoryGirl.build :original_picture, :artist_id => @artist.id, :stopped_at => Time.now + 400
        end
        it '素材に停止を依頼している' do
          ResourcePicture.any_instance.should_receive(:unpublish).with(any_args).exactly(1)
          r = ProviderOriginalPicture.unpublish_resource_picture @pop, @stopped_op_attr
        end
      end
      context '対照表から原画を取得できないとき' do
        before do
          @pop = ProviderOriginalPicture.new
        end
        it '素材に停止を依頼していない' do
          ResourcePicture.any_instance.should_not_receive(:unpublish).with(any_args)
          r = ProviderOriginalPicture.unpublish_resource_picture @pop, @stopped_op_attr
        end
      end
      context '対照表から素材を取得できないとき' do
        before do
          @rp.destroy
        end
        it '素材に停止を依頼していない' do
          ResourcePicture.any_instance.should_not_receive(:unpublish).with(any_args)
          r = ProviderOriginalPicture.unpublish_resource_picture @pop, @stopped_op_attr
        end
      end
    end
    context '貸手側原画のカラム値の停止日時が設定されていないとき' do
      before do
        @pop = ProviderOriginalPicture.create :provider_id => @provider.id, :providers_original_picture_id => 1, :demanders_original_picture_id => @op.id
        @stopped_op = FactoryGirl.build :original_picture, :artist_id => @artist.id, :stopped_at => nil
        @stopped_op_attr = @stopped_op.attributes
        ResourcePicture.any_instance.stub(:unpublish).with(any_args).and_return(true)
      end
      it '素材に停止を依頼していない' do
        ResourcePicture.any_instance.should_not_receive(:unpublish).with(any_args)
        r = ProviderOriginalPicture.unpublish_resource_picture @pop, @stopped_op_attr
      end
    end
    context 'つつがなく終わるとき' do
      before do
        @pop = ProviderOriginalPicture.create :provider_id => @provider.id, :providers_original_picture_id => 1, :demanders_original_picture_id => @op.id
        @stopped_op = FactoryGirl.build :original_picture, :artist_id => @artist.id, :stopped_at => Time.now + 400
        @stopped_op_attr = @stopped_op.attributes
        ResourcePicture.any_instance.stub(:unpublish).with(any_args).and_return(true)
      end
      it 'trueを返す' do
        r = ProviderOriginalPicture.unpublish_resource_picture @pop, @stopped_op_attr
        r.should be_true
      end
    end
    context '素材停止に失敗したとき' do
      before do
        @pop = ProviderOriginalPicture.create :provider_id => @provider.id, :providers_original_picture_id => 1, :demanders_original_picture_id => @op.id
        @stopped_op = FactoryGirl.build :original_picture, :artist_id => @artist.id, :stopped_at => Time.now + 400
        @stopped_op_attr = @stopped_op.attributes
        ResourcePicture.any_instance.stub(:unpublish).with(any_args).and_return(false)
      end
      it 'falseを返す' do
        r = ProviderOriginalPicture.unpublish_resource_picture @pop, @stopped_op_attr
        r.should be_false
      end
    end
  end
  
  describe '原画インポートに於いて' do
    before do
      @op = FactoryGirl.create :original_picture, :artist_id => @artist.id
      @other_op = FactoryGirl.create :original_picture, :artist_id => @artist.id
    end
    context '事前チェックする' do
      before do
        ProviderOriginalPicture.stub(:get_one).with(any_args).and_return(ProviderOriginalPicture.new)
        ProviderOriginalPicture.stub(:replace_artist).with(any_args).and_return(true)
        ProviderOriginalPicture.stub(:unpublish_resource_picture).with(any_args).and_return(true)
        ProviderOriginalPicture.any_instance.stub(:modify_original_picture).with(any_args).and_return(@op)
        OriginalPicture.any_instance.stub(:valid?).with(any_args).and_return(true)
        ProviderOriginalPicture.any_instance.stub(:save).with(any_args).and_return(true)
      end
      it '対照表取得を問い合わせている' do
        ProviderOriginalPicture.should_receive(:get_one).with(any_args).exactly(1)
        r = ProviderOriginalPicture.import_original_pictures @provider.id, [@op.attributes]
      end
      it '絵師id置換を依頼している' do
        ProviderOriginalPicture.should_receive(:replace_artist).with(any_args).exactly(1)
        r = ProviderOriginalPicture.import_original_pictures @provider.id, [@op.attributes]
      end
      it '素材停止を依頼している' do
        ProviderOriginalPicture.should_receive(:unpublish_resource_picture).with(any_args).exactly(1)
        r = ProviderOriginalPicture.import_original_pictures @provider.id, [@op.attributes]
      end
      it '対照表に原画同期を依頼している' do
        ProviderOriginalPicture.any_instance.should_receive(:modify_original_picture).with(any_args).exactly(1)
        r = ProviderOriginalPicture.import_original_pictures @provider.id, [@op.attributes]
      end
      it '原画を検証している' do
        OriginalPicture.any_instance.should_receive(:valid?).with(any_args).exactly(1)
        r = ProviderOriginalPicture.import_original_pictures @provider.id, [@op.attributes]
      end
      it '対照表オブジェクトを保存している' do
        ProviderOriginalPicture.any_instance.should_receive(:save).with(any_args).exactly(1)
        r = ProviderOriginalPicture.import_original_pictures @provider.id, [@op.attributes]
      end
    end
    context 'つつがなく終わるとき' do
      before do
        ProviderOriginalPicture.stub(:replace_artist).with(any_args).and_return(true)
      end
      it '空っぽの配列を返す' do
        r = ProviderOriginalPicture.import_original_pictures @provider.id, [@op.attributes]
        r.should be_empty
      end
      it '対照表が追加される' do
        lambda {
          r = ProviderOriginalPicture.import_original_pictures @provider.id, [@op.attributes]
        }.should change ProviderOriginalPicture, :count
      end
    end
    context '複数インポートのとき' do
      before do
        #前回インポートで貸手からもらった素材
        @other_p = FactoryGirl.create :picture, :original_picture_id => @other_op.id, :license_id => @license.id, :artist_id => @artist.id
        rpattr = {:original_picture_id => @other_op.id, :picture_id => @other_p.id, :license_id => @license.id, :artist_id => @artist.id, :artist_name => 'tester', 
          :credit => '{}', :settings => {:reverse => 1, :gif_convert => 1}.to_json.to_s}
        @other_rp = FactoryGirl.create :resource_picture, rpattr
        #対照表も用意されている
        @pop = ProviderOriginalPicture.create :provider_id => @provider.id, :providers_original_picture_id => @other_op.id, :demanders_original_picture_id => @other_op.id
        #これが今回停止される
        @other_op_attr = @other_op.attributes
        @other_op_attr['stopped_at'] = Time.now - 100
        @other_op_attr['created_at'] = Time.now - 100
        @other_op_attr['updated_at'] = Time.now - 100
        ProviderOriginalPicture.stub(:replace_artist).with(any_args).and_return(true)
      end
      it '空っぽの配列を返す' do
        r = ProviderOriginalPicture.import_original_pictures @provider.id, [@op.attributes, @other_op_attr]
        r.should be_empty
      end
      it '新規分の対照表が追加される' do
        lambda {
          r = ProviderOriginalPicture.import_original_pictures @provider.id, [@op.attributes, @other_op_attr]
        }.should change(ProviderOriginalPicture, :count)
      end
      it '停止された素材が削除される' do
        lambda {
          r = ProviderOriginalPicture.import_original_pictures @provider.id, [@op.attributes, @other_op_attr]
        }.should change(ResourcePicture, :count)
      end
      it '原画の日時は貸手と一致しており、素材停止の影響を受けていない' do
        r = ProviderOriginalPicture.import_original_pictures @provider.id, [@op.attributes, @other_op_attr]
        @other_op.reload
        @other_op.updated_at.should eq @other_op_attr['updated_at']
      end
    end
    #警告ケース
    context '絵師id置換に失敗したとき' do
      before do
        ProviderOriginalPicture.stub(:replace_artist).with(any_args).and_return(false)
      end
      it '結果に原画のカラム値を追加している' do
        r = ProviderOriginalPicture.import_original_pictures @provider.id, [@op.attributes, @op.attributes]
        r.should_not be_empty
      end
    end
    context '原画の検証に失敗したとき' do
      before do
        OriginalPicture.any_instance.stub(:valid?).with(any_args).and_return(false)
      end
     it '結果に貸手側原画のカラム値を追加している' do
        r = ProviderOriginalPicture.import_original_pictures @provider.id, [@op.attributes]
        r.should_not be_empty
      end
    end
    context '対照表オブジェクトの保存に失敗したとき' do
      before do
        ProviderOriginalPicture.any_instance.stub(:save).with(any_args).and_return(false)
      end
     it '結果に貸手側原画のカラム値を追加している' do
        r = ProviderOriginalPicture.import_original_pictures @provider.id, [@op.attributes]
        r.should_not be_empty
      end
    end
    context '原画オブジェクトの保存に失敗したとき' do
      before do
        OriginalPicture.any_instance.stub(:save).with(any_args).and_return(false)
      end
      it '結果に貸手側原画のカラム値を追加している' do
        r = ProviderOriginalPicture.import_original_pictures @provider.id, [@op.attributes]
        r.should_not be_empty
      end
    end
  end
  
  describe '実素材インポートに於いて' do
    before do
      @op = FactoryGirl.create :original_picture, :artist_id => @artist.id
      @p = FactoryGirl.create :picture, :original_picture_id => @op.id, :license_id => @license.id, :artist_id => @artist.id
      @imager = ImagerTest.load "abc\ndef\nghi"
      @attr = @p.attributes
      @attr['picture_data'] =  "abc\ndef\nghi"
    end
    context '事前チェックする' do
      before do
        ProviderOriginalPicture.stub(:replace_original_picture).with(any_args).and_return(true)
        ProviderOriginalPicture.stub(:replace_artist).with(any_args).and_return(true)
        ProviderOriginalPicture.stub(:replace_license).with(any_args).and_return(true)
        Base64.stub(:decode64).with(any_args).and_return("abc\ndef\nghi")
        PettanImager.stub(:load).with("abc\ndef\nghi").and_return(@imager)
        ProviderOriginalPicture.stub(:modify_picture).with(any_args).and_return(@p)
        Picture.any_instance.stub(:store).with(any_args).and_return(true)
      end
      it '原画id置換を依頼している' do
        ProviderOriginalPicture.should_receive(:replace_original_picture).with(any_args).exactly(1)
        r = ProviderOriginalPicture.import_pictures @provider.id, [@attr]
      end
      it '絵師id置換を依頼している' do
        ProviderOriginalPicture.should_receive(:replace_artist).with(any_args).exactly(1)
        r = ProviderOriginalPicture.import_pictures @provider.id, [@attr]
      end
      it 'ライセンスid置換を依頼している' do
        ProviderOriginalPicture.should_receive(:replace_license).with(any_args).exactly(1)
        r = ProviderOriginalPicture.import_pictures @provider.id, [@attr]
      end
      it '画像データをデコードしている' do
        Base64.should_receive(:decode64).with(any_args).exactly(1)
        r = ProviderOriginalPicture.import_pictures @provider.id, [@attr]
      end
      it '画像ライブラリにロードを依頼している' do
        PettanImager.should_receive(:load).with(any_args).exactly(1)
        r = ProviderOriginalPicture.import_pictures @provider.id, [@attr]
      end
      it '対照表モデルに貸手側実素材同期を依頼している' do
        ProviderOriginalPicture.should_receive(:modify_picture).with(any_args).exactly(1)
        r = ProviderOriginalPicture.import_pictures @provider.id, [@attr]
      end
      it '実素材を保存している' do
        Picture.any_instance.should_receive(:store).with(any_args).exactly(1)
        r = ProviderOriginalPicture.import_pictures @provider.id, [@attr]
      end
    end
    context 'つつがなく終わるとき' do
      before do
        ProviderOriginalPicture.stub(:replace_original_picture).with(any_args).and_return(true)
        ProviderOriginalPicture.stub(:replace_artist).with(any_args).and_return(true)
        ProviderOriginalPicture.stub(:replace_license).with(any_args).and_return(true)
        Base64.stub(:decode64).with(any_args).and_return("abc\ndef\nghi")
        PettanImager.stub(:load).with("abc\ndef\nghi").and_return(@imager)
        @newp = FactoryGirl.build :picture, :original_picture_id => @op.id, :license_id => @license.id, :artist_id => @artist.id, :revision => 1
        @newattr = @newp.attributes
        @newattr['picture_data'] =  "abc\ndef\nghi"
      end
      it '空っぽの配列を返す' do
        r = ProviderOriginalPicture.import_pictures @provider.id, [@newattr]
        r.should be_empty
      end
      it '実素材が追加される' do
        lambda {
          r = ProviderOriginalPicture.import_pictures @provider.id, [@newattr]
        }.should change Picture, :count
      end
    end
    #警告ケース
    context '原画id置換に失敗したとき' do
      before do
        ProviderOriginalPicture.stub(:replace_original_picture).with(any_args).and_return(false)
      end
      it '結果に実素材のカラム値を追加している' do
        r = ProviderOriginalPicture.import_pictures @provider.id, [@attr]
        r.should_not be_empty
      end
    end
    context '絵師id置換に失敗したとき' do
      before do
        ProviderOriginalPicture.stub(:replace_original_picture).with(any_args).and_return(true)
        ProviderOriginalPicture.stub(:replace_artist).with(any_args).and_return(false)
      end
      it '結果に実素材のカラム値を追加している' do
        r = ProviderOriginalPicture.import_pictures @provider.id, [@attr]
        r.should_not be_empty
      end
    end
    context 'ライセンスid置換に失敗したとき' do
      before do
        ProviderOriginalPicture.stub(:replace_original_picture).with(any_args).and_return(true)
        ProviderOriginalPicture.stub(:replace_artist).with(any_args).and_return(true)
        ProviderOriginalPicture.stub(:replace_license).with(any_args).and_return(false)
      end
      it '結果に実素材のカラム値を追加している' do
        r = ProviderOriginalPicture.import_pictures @provider.id, [@attr]
        r.should_not be_empty
      end
    end
    context '画像ライブラリのロードに失敗したとき' do
      before do
        ProviderOriginalPicture.stub(:replace_original_picture).with(any_args).and_return(true)
        ProviderOriginalPicture.stub(:replace_artist).with(any_args).and_return(true)
        ProviderOriginalPicture.stub(:replace_license).with(any_args).and_return(true)
        PettanImager.stub(:load).with(any_args).and_return(false)
      end
      it '結果に実素材のカラム値を追加している' do
        r = ProviderOriginalPicture.import_pictures @provider.id, [@attr]
        r.should_not be_empty
      end
    end
    context '実素材の保存に失敗したとき' do
      before do
        ProviderOriginalPicture.stub(:replace_original_picture).with(any_args).and_return(true)
        ProviderOriginalPicture.stub(:replace_artist).with(any_args).and_return(true)
        ProviderOriginalPicture.stub(:replace_license).with(any_args).and_return(true)
        PettanImager.stub(:load).with(any_args).and_return(@imager)
        ProviderOriginalPicture.stub(:modify_picture).with(any_args).and_return(@p)
        Picture.any_instance.stub(:store).with(any_args).and_return(false)
      end
      it '結果に実素材のカラム値を追加している' do
        r = ProviderOriginalPicture.import_pictures @provider.id, [@attr]
        r.should_not be_empty
      end
    end
  end
  
  describe '素材インポートに於いて' do
    before do
      @op = FactoryGirl.create :original_picture, :artist_id => @artist.id
      @p = FactoryGirl.create :picture, :artist_id => @artist.id, :original_picture_id => @op.id, :license_id => @license.id, :revision => 0
      rpattr = {:original_picture_id => @op.id, :picture_id => @p.id, :license_id => @license.id, :artist_id => @artist.id, :artist_name => 'tester', 
        :credit => '{}', :settings => {:reverse => 1, :gif_convert => 1}.to_json.to_s}
      @rp = FactoryGirl.create :resource_picture, rpattr
      @imager = ImagerTest.load "abc\ndef\nghi"
      @attr = @op.attributes
      @attr['resource_picture'] =  @rp.attributes
      @attr['resource_picture']['picture_data'] =  "abc\ndef\nghi"
    end
    context '事前チェックする' do
      before do
        ProviderOriginalPicture.stub(:replace_original_picture).with(any_args).and_return(true)
        ProviderOriginalPicture.stub(:replace_artist).with(any_args).and_return(true)
        ProviderOriginalPicture.stub(:replace_license).with(any_args).and_return(true)
        ProviderOriginalPicture.stub(:replace_picture).with(any_args).and_return(true)
        Base64.stub(:decode64).with(any_args).and_return("abc\ndef\nghi")
        PettanImager.stub(:load).with("abc\ndef\nghi").and_return(@imager)
        ProviderOriginalPicture.stub(:modify_resource_picture).with(any_args).and_return(@rp)
        ResourcePicture.any_instance.stub(:store_picture_with_gif).with(any_args).and_return(true)
        ResourcePicture.any_instance.stub(:save).with(any_args).and_return(true)
      end
      it '原画id置換を依頼している' do
        ProviderOriginalPicture.should_receive(:replace_original_picture).with(any_args).exactly(1)
        r = ProviderOriginalPicture.import_resource_pictures @provider.id, [@attr]
      end
      it '絵師id置換を依頼している' do
        ProviderOriginalPicture.should_receive(:replace_artist).with(any_args).exactly(1)
        r = ProviderOriginalPicture.import_resource_pictures @provider.id, [@attr]
      end
      it 'ライセンスid置換を依頼している' do
        ProviderOriginalPicture.should_receive(:replace_license).with(any_args).exactly(1)
        r = ProviderOriginalPicture.import_resource_pictures @provider.id, [@attr]
      end
      it '実素材id置換を依頼している' do
        ProviderOriginalPicture.should_receive(:replace_picture).with(any_args).exactly(1)
        r = ProviderOriginalPicture.import_resource_pictures @provider.id, [@attr]
      end
      it '画像データをデコードしている' do
        Base64.should_receive(:decode64).with(any_args).exactly(1)
        r = ProviderOriginalPicture.import_resource_pictures @provider.id, [@attr]
      end
      it '画像ライブラリにロードを依頼している' do
        PettanImager.should_receive(:load).with(any_args).exactly(1)
        r = ProviderOriginalPicture.import_resource_pictures @provider.id, [@attr]
      end
      it '対照表モデルに貸手側素材同期を依頼している' do
        ProviderOriginalPicture.should_receive(:modify_resource_picture).with(any_args).exactly(1)
        r = ProviderOriginalPicture.import_resource_pictures @provider.id, [@attr]
      end
      it '素材を保存している' do
        ResourcePicture.any_instance.should_receive(:save).with(any_args).exactly(1)
        r = ProviderOriginalPicture.import_resource_pictures @provider.id, [@attr]
      end
      it '素材にgif付き画像ファイルの作成・更新を依頼している' do
        ResourcePicture.any_instance.should_receive(:store_picture_with_gif).with(any_args).exactly(1)
        r = ProviderOriginalPicture.import_resource_pictures @provider.id, [@attr]
      end
      context '素材のカラム値を取得できないとき' do
        before do
          @attr['resource_picture'] =  nil
          ProviderOriginalPicture.stub(:import_resource_picture).with(any_args).and_return(nil)
        end
        it '素材インポートを依頼していない' do
          ProviderOriginalPicture.should_not_receive(:import_resource_picture).with(any_args)
          r = ProviderOriginalPicture.import_resource_pictures @provider.id, [@attr]
        end
      end
    end
    context 'つつがなく終わるとき' do
      before do
        ProviderOriginalPicture.stub(:replace_original_picture).with(any_args).and_return(true)
        ProviderOriginalPicture.stub(:replace_artist).with(any_args).and_return(true)
        ProviderOriginalPicture.stub(:replace_license).with(any_args).and_return(true)
        Base64.stub(:decode64).with(any_args).and_return("abc\ndef\nghi")
        PettanImager.stub(:load).with("abc\ndef\nghi").and_return(@imager)
        @op2 = FactoryGirl.create :original_picture, :artist_id => @artist.id
        @p2 = FactoryGirl.create :picture, :artist_id => @artist.id, :original_picture_id => @op2.id, :license_id => @license.id, :revision => 0
        newrpattr = {:original_picture_id => @op2.id, :picture_id => @p2.id, :license_id => @license.id, :artist_id => @artist.id, :artist_name => 'tester', 
          :credit => '{}', :settings => {:reverse => 1, :gif_convert => 1, :thumbnail => 0}.to_json.to_s}
        @rp2 = FactoryGirl.build :resource_picture, newrpattr
        @newattr = @op2.attributes
        @newattr['resource_picture'] =  @rp2.attributes
        @newattr['resource_picture']['picture_data'] =  "abc\ndef\nghi"
      end
      it '空っぽの配列を返す' do
        r = ProviderOriginalPicture.import_resource_pictures @provider.id, [@newattr]
        r.should be_empty
      end
      it '素材が追加される' do
        lambda {
          r = ProviderOriginalPicture.import_resource_pictures @provider.id, [@newattr]
        }.should change ResourcePicture, :count
      end
    end
    #警告ケース
    context '原画id置換に失敗したとき' do
      before do
        ProviderOriginalPicture.stub(:replace_original_picture).with(any_args).and_return(false)
      end
      it '結果に素材のカラム値を追加している' do
        r = ProviderOriginalPicture.import_resource_pictures @provider.id, [@attr]
        r.should_not be_empty
      end
    end
    context '絵師id置換に失敗したとき' do
      before do
        ProviderOriginalPicture.stub(:replace_original_picture).with(any_args).and_return(true)
        ProviderOriginalPicture.stub(:replace_artist).with(any_args).and_return(false)
      end
      it '結果に素材のカラム値を追加している' do
        r = ProviderOriginalPicture.import_resource_pictures @provider.id, [@attr]
        r.should_not be_empty
      end
    end
    context 'ライセンスid置換に失敗したとき' do
      before do
        ProviderOriginalPicture.stub(:replace_original_picture).with(any_args).and_return(true)
        ProviderOriginalPicture.stub(:replace_artist).with(any_args).and_return(true)
        ProviderOriginalPicture.stub(:replace_license).with(any_args).and_return(false)
      end
      it '結果に素材のカラム値を追加している' do
        r = ProviderOriginalPicture.import_resource_pictures @provider.id, [@attr]
        r.should_not be_empty
      end
    end
    context '実素材id置換に失敗したとき' do
      before do
        ProviderOriginalPicture.stub(:replace_original_picture).with(any_args).and_return(true)
        ProviderOriginalPicture.stub(:replace_artist).with(any_args).and_return(true)
        ProviderOriginalPicture.stub(:replace_license).with(any_args).and_return(true)
        ProviderOriginalPicture.stub(:replace_picture).with(any_args).and_return(false)
      end
      it '結果に素材のカラム値を追加している' do
        r = ProviderOriginalPicture.import_resource_pictures @provider.id, [@attr]
        r.should_not be_empty
      end
    end
    context '画像ライブラリのロードに失敗したとき' do
      before do
        ProviderOriginalPicture.stub(:replace_original_picture).with(any_args).and_return(true)
        ProviderOriginalPicture.stub(:replace_artist).with(any_args).and_return(true)
        ProviderOriginalPicture.stub(:replace_license).with(any_args).and_return(true)
        PettanImager.stub(:load).with(any_args).and_return(false)
      end
      it '結果に素材のカラム値を追加している' do
        r = ProviderOriginalPicture.import_resource_pictures @provider.id, [@attr]
        r.should_not be_empty
      end
    end
    context '素材の保存に失敗したとき' do
      before do
        ProviderOriginalPicture.stub(:replace_original_picture).with(any_args).and_return(true)
        ProviderOriginalPicture.stub(:replace_artist).with(any_args).and_return(true)
        ProviderOriginalPicture.stub(:replace_license).with(any_args).and_return(true)
        PettanImager.stub(:load).with(any_args).and_return(@imager)
        ProviderOriginalPicture.stub(:modify_resource_picture).with(any_args).and_return(@rp)
        ResourcePicture.any_instance.stub(:store_picture_with_gif).with(any_args).and_return(true)
        ResourcePicture.any_instance.stub(:save).with(any_args).and_return(false)
      end
      it '結果に素材のカラム値を追加している' do
        r = ProviderOriginalPicture.import_resource_pictures @provider.id, [@attr]
        r.should_not be_empty
      end
    end
    context 'gif付き画像ファイルの作成・更新に失敗したとき' do
      before do
        ProviderOriginalPicture.stub(:replace_original_picture).with(any_args).and_return(true)
        ProviderOriginalPicture.stub(:replace_artist).with(any_args).and_return(true)
        ProviderOriginalPicture.stub(:replace_license).with(any_args).and_return(true)
        PettanImager.stub(:load).with(any_args).and_return(@imager)
        ProviderOriginalPicture.stub(:modify_resource_picture).with(any_args).and_return(@rp)
        ResourcePicture.any_instance.stub(:store_picture_with_gif).with(any_args).and_return(false)
      end
      it '結果に素材のカラム値を追加している' do
        r = ProviderOriginalPicture.import_resource_pictures @provider.id, [@attr]
        r.should_not be_empty
      end
    end
  end
  
  describe 'インポートに於いて' do
    before do
      @op = FactoryGirl.create :original_picture, :artist_id => @artist.id
      @p = FactoryGirl.create :picture, :artist_id => @artist.id, :original_picture_id => @op.id, :license_id => @license.id, :revision => 0
      @opattr = @op.attributes
      @pattr = @p.attributes
      @pattr['picture_data'] =  "abc\ndef\nghi"
      @rpattr = {:original_picture_id => @op.id, :picture_id => @p.id, :license_id => @license.id, :artist_id => @artist.id, :artist_name => 'tester', 
        :credit => '{}', :settings => {:reverse => 1, :gif_convert => 1}.to_json.to_s}
      @opattr['resource_picture'] =  @rpattr
      @opattr['resource_picture']['picture_data'] =  "abc\ndef\nghi"
    end
    context '事前チェックする' do
      before do
        ProviderOriginalPicture.stub(:import_original_pictures).with(any_args).and_return([])
        ProviderOriginalPicture.stub(:import_pictures).with(any_args).and_return([])
        ProviderOriginalPicture.stub(:import_resource_pictures).with(any_args).and_return([])
      end
      it '自身に原画インポートを依頼している' do
        ProviderOriginalPicture.should_receive(:import_original_pictures).with(any_args).exactly(1)
        r = ProviderOriginalPicture.import @provider.id, [@opattr], [@pattr]
      end
      it '自身に実素材インポートを依頼している' do
        ProviderOriginalPicture.should_receive(:import_pictures).with(any_args).exactly(1)
        r = ProviderOriginalPicture.import @provider.id, [@opattr], [@pattr]
      end
      it '自身に素材インポートを依頼している' do
        ProviderOriginalPicture.should_receive(:import_resource_pictures).with(any_args).exactly(1)
        r = ProviderOriginalPicture.import @provider.id, [@opattr], [@pattr]
      end
    end
    context 'つつがなく終わるとき' do
      before do
        ProviderOriginalPicture.stub(:import_original_pictures).with(any_args).and_return([])
        ProviderOriginalPicture.stub(:import_pictures).with(any_args).and_return([])
        ProviderOriginalPicture.stub(:import_resource_pictures).with(any_args).and_return([])
      end
      it 'ImportResultを返す' do
        r = ProviderOriginalPicture.import @provider.id, [@opattr], [@pattr]
        r.is_a?(ImportResult).should be_true
        r.should be_success
      end
    end
    #警告ケース
    context '原画インポートに失敗したとき' do
      before do
        ProviderOriginalPicture.stub(:import_original_pictures).with(any_args).and_return([@opattr])
        ProviderOriginalPicture.stub(:import_pictures).with(any_args).and_return([])
        ProviderOriginalPicture.stub(:import_resource_pictures).with(any_args).and_return([])
      end
      it '結果が失敗している' do
        r = ProviderOriginalPicture.import @provider.id, [@opattr], [@pattr]
        r.should_not be_success
        r.is_a?(OriginalPictureImportResult).should be_true
        r.failures.should_not be_empty
      end
    end
    context '実素材インポートに失敗したとき' do
      before do
        ProviderOriginalPicture.stub(:import_original_pictures).with(any_args).and_return([])
        ProviderOriginalPicture.stub(:import_pictures).with(any_args).and_return([@pattr])
        ProviderOriginalPicture.stub(:import_resource_pictures).with(any_args).and_return([])
      end
      it '結果が失敗している' do
        r = ProviderOriginalPicture.import @provider.id, [@opattr], [@pattr]
        r.should_not be_success
        r.is_a?(PictureImportResult).should be_true
        r.failures.should_not be_empty
      end
    end
    context '素材インポートに失敗したとき' do
      before do
        ProviderOriginalPicture.stub(:import_original_pictures).with(any_args).and_return([])
        ProviderOriginalPicture.stub(:import_pictures).with(any_args).and_return([])
        ProviderOriginalPicture.stub(:import_resource_pictures).with(any_args).and_return([@rpattr])
      end
      it '結果が失敗している' do
        r = ProviderOriginalPicture.import @provider.id, [@opattr], [@pattr]
        r.should_not be_success
        r.is_a?(ResourcePictureImportResult).should be_true
        r.failures.should_not be_empty
      end
    end
  end
  

end
